public class JosephusListe<T> {

    private class Item {
        T    value;		// im Hilfsobjekt gespeicherter Wert
        Item prev;		// naechstes Hilfsobjekt in der Liste
        Item next;		// vorangehendes Hilfsobjekt in der Liste
    }

    private int  size;		// Listengroesse (=Anzahl der Objekte)
    private Item cursor;	// aktuelles Hilfsobjekt 

    public JosephusListe() {
        size = 0;
        cursor = null;
    }

    public void insert(T value) {	// es wird immer hinter cursor eingefuegt
       Item item = new Item();

       item.value = value;

       if ( cursor == null ) {  	// Liste ist bisher leer,
           item.next = item;    	//  dann ein Element, das auf sich selber zeigt
	   item.prev = item;
       }
       else {				// ansonsten
           item.next = cursor.next;	// ist das Objekt hinter dem Cursor der Nachfolger und
	   item.prev = cursor;		// Cursor ist der Vorgaenger

	   cursor.next.prev = item;	// im Nachfolger den Vorgaenger aendern
	   cursor.next = item;		// im Vorgaenger den Nachfolger aendern
       }
       cursor = item;			// das eingefuegte Objekt ist jetzt das aktuelle

       size++;
    }

    public void delete() {		// aktuelles Objekt loeschen
        Item pred;
	Item succ;

        if ( cursor == null ) {
            throw new IllegalStateException("JosephusListe.delete(): Liste ist leer");
        }

	if ( cursor == cursor.next ) {	// Liste hat nur noch ein Element
	    cursor = null;		// dann ist sie jetzt leer
	}
        else {
	    pred = cursor.prev;		// Vorgaenger des zu loeschenden Elements
	    succ = cursor.next;		// Nachfolger

	    pred.next = succ;		// Zeiger umhaengen
	    succ.prev = pred;

	    cursor = cursor.next;	// Nachfolger ist neues akteulles Element
	}

        size--;
    }

    public int size() {
        return size;
    }

    public T get() {
        return cursor.value;
    }

    public void forward(int steps) {
        for (int i=0 ; i<steps ; i++) {
	    cursor = cursor.next;
	}
    }

    public void backward(int steps) {
        for (int i=0 ; i<steps ; i++) {
	    cursor = cursor.prev;
	}
    }
}
